==> microfrontends-0.0.10.tar.gz: lib/src/async.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:microfrontends/microfrontends.dart';

import 'mutation.dart';
import 'resolver.dart';
import 'state_container.dart';

abstract class Async<T> {}

class Uninitialized<T> extends Async<T> {}

class Loading<T> extends Async<T> {}

class Loaded<T> extends Async<T> {
  final T state;
  Loaded(this.state);
}

class Errored<T> extends Async<T> {
  final Exception exception;

  Errored(this.exception);
}

class AsyncResolverController<T> extends ResolverController<Async<T>> {
  AsyncResolverController(ValueNotifier<Async<T>> stateNotifier)
      : super(stateNotifier);

  void load(Future<T> Function() executor) {
    run(() async* {
      yield Loading<T>();
      final d = await executor();
      yield Loaded(d);
    });
  }
}

typedef AsyncResolverCallback<R, S, U> = Async<R> Function(
  AsyncResolverController<S> ctrl,
  Async<S> value,
  U? arg,
);

class AsyncMutationResolver<T extends Mutation, K>
    extends MutationResolver<T, Async<K>> {
  AsyncMutationResolver(
      AsyncResolverController<K> ctrl, AsyncResolverCallback<K, K, T> callback)
      : super(ctrl, (a, b, c) {
          return callback(a as AsyncResolverController<K>, b, c);
        });
}

class NotImplemented implements Exception {}

extension AsyncCast<T> on Future<T> {
  Future<Async<T>> toAsync() {
    return this
        .then((value) => Loaded<T>(value))
        .catchError((err) => throw Errored<T>(err));
  }
}

extension StateCast<T> on T {
  Loaded<T> ok() {
    return Loaded(this);
  }
}

extension AsyncError on Exception {
  Errored<T> err<T>() {
    return Errored<T>(this);
  }
}

class AsyncStateContainer<T> extends StateContainer<Async<T>> {
  Async<T> get initialState => Uninitialized<T>();

  Stream<Async<T>> init(context) async* {
    Future<Async<T>> req;

    try {
      req = load();
    } on NotImplemented {
      return;
    }

    yield Loading<T>();

    try {
      final d = await req;
      yield d;
    } on Exception catch (err) {
      yield Errored<T>(err);
    } on Errored<T> catch (err) {
      yield err;
    }
  }

  Future<Async<T>> load() {
    throw new NotImplemented();
  }

  @override
  AsyncResolverController<T> get resolverController =>
      AsyncResolverController(stateNotifier);

  @override
  void mutation<K extends Mutation>(AsyncResolverCallback<T, T, K> callback) {
    final mutationResolver =
        AsyncMutationResolver<K, T>(resolverController, callback);
    mutationResolvers[K] = mutationResolver;
  }
}

class CaseAsyncType<T> extends CaseType<Async<T>> {
  void uninitialized(StateTypeMatchBuilder<Uninitialized<T>> builder) {
    return super.matches<Uninitialized<T>>(builder);
  }

  void loading(StateTypeMatchBuilder<Loading<T>> builder) {
    return super.matches<Loading<T>>(builder);
  }

  void loaded(StateTypeMatchBuilder<Loaded<T>> builder) {
    return super.matches<Loaded<T>>(builder);
  }

  void errored(StateTypeMatchBuilder<Errored<T>> builder) {
    return super.matches<Errored<T>>(builder);
  }
}

class FutureStateContainer<T> extends AsyncStateContainer<T> {
  final Future<T> future;

  FutureStateContainer(this.future);

  @override
  Future<Async<T>> load() {
    return future.toAsync();
  }
}

class StateStream<T> extends AsyncStateContainer<T> {
  final Stream<T> stream;
  StateStream(this.stream);

  Stream<T> run() => stream;
}

@immutable
abstract class StateContainerFactoryWidget<T> extends Widget {
  const StateContainerFactoryWidget({Key? key}) : super(key: key);

  StateContainer<T> _createContainer();

  @override
  Element createElement() {
    return StateContainerFactoryElement<T>(this);
  }

  Widget build(BuildContext context);
}

class StateContainerFactoryElement<T> extends ComponentElement {
  StateContainerFactoryElement(Widget widget) : super(widget);

  StateContainerFactoryWidget<T> get widget =>
      super.widget as StateContainerFactoryWidget<T>;
  late final StateContainer<T> container;

  @override
  void mount(Element? parent, Object? newSlot) {
    container = widget._createContainer();
    super.mount(parent, slot);
  }

  @override
  Widget build() {
    return StaticContainer(
      container: container,
      child: widget.build(this),
    );
  }
}

abstract class FutureStateWidget<T>
    extends StateContainerFactoryWidget<Async<T>> {
  const FutureStateWidget({Key? key}) : super(key: key);

  Future<T> get future;

  @override
  AsyncStateContainer<T> _createContainer() {
    return FutureStateContainer<T>(future);
  }
}

abstract class StateStreamWidget<T>
    extends StateContainerFactoryWidget<Async<T>> {
  const StateStreamWidget({Key? key}) : super(key: key);
  Stream<T> get stream;

  @override
  AsyncStateContainer<T> _createContainer() {
    return StateStream<T>(stream);
  }
}

==> microfrontends-0.0.11.tar.gz: lib/src/async.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:microfrontends/microfrontends.dart';

import 'mutation.dart';
import 'resolver.dart';
import 'state_container.dart';

abstract class Async<T> {}

class Uninitialized<T> extends Async<T> {}

class Loading<T> extends Async<T> {}

class Loaded<T> extends Async<T> {
  final T state;
  Loaded(this.state);
}

class Errored<T> extends Async<T> {
  final Exception exception;

  Errored(this.exception);
}

class AsyncResolverController<T> extends ResolverController<Async<T>> {
  AsyncResolverController(ValueNotifier<Async<T>> stateNotifier)
      : super(stateNotifier);

  void load(Future<T> Function() executor) {
    run(() async* {
      yield Loading<T>();
      final d = await executor();
      yield Loaded(d);
    });
  }
}

typedef AsyncResolverCallback<R, S, U> = Async<R> Function(
  AsyncResolverController<S> ctrl,
  Async<S> value,
  U? arg,
);

class AsyncMutationResolver<T extends Mutation, K>
    extends MutationResolver<T, Async<K>> {
  AsyncMutationResolver(
      AsyncResolverController<K> ctrl, AsyncResolverCallback<K, K, T> callback)
      : super(ctrl, (a, b, c) {
          return callback(a as AsyncResolverController<K>, b, c);
        });
}

class NotImplemented implements Exception {}

extension AsyncCast<T> on Future<T> {
  Future<Async<T>> toAsync() {
    return this
        .then((value) => Loaded<T>(value))
        .catchError((err) => throw Errored<T>(err));
  }
}

extension StateCast<T> on T {
  Loaded<T> ok() {
    return Loaded(this);
  }
}

extension AsyncError on Exception {
  Errored<T> err<T>() {
    return Errored<T>(this);
  }
}

class AsyncStateContainer<T> extends StateContainer<Async<T>> {
  Async<T> get initialState => Uninitialized<T>();

  Stream<Async<T>> init(context) async* {
    Future<Async<T>> req;

    try {
      req = load();
    } on NotImplemented {
      return;
    }

    yield Loading<T>();

    try {
      final d = await req;
      yield d;
    } on Exception catch (err) {
      yield Errored<T>(err);
    } on Errored<T> catch (err) {
      yield err;
    }
  }

  Future<Async<T>> load() {
    throw new NotImplemented();
  }

  @override
  AsyncResolverController<T> get resolverController =>
      AsyncResolverController(stateNotifier);

  @override
  void mutation<K extends Mutation>(AsyncResolverCallback<T, T, K> callback) {
    final mutationResolver =
        AsyncMutationResolver<K, T>(resolverController, callback);
    mutationResolvers[K] = mutationResolver;
  }
}

class CaseAsyncType<T> extends CaseType<Async<T>> {
  void uninitialized(StateTypeMatchBuilder<Uninitialized<T>> builder) {
    return super.matches<Uninitialized<T>>(builder);
  }

  void loading(StateTypeMatchBuilder<Loading<T>> builder) {
    return super.matches<Loading<T>>(builder);
  }

  void loaded(StateTypeMatchBuilder<Loaded<T>> builder) {
    return super.matches<Loaded<T>>(builder);
  }

  void errored(StateTypeMatchBuilder<Errored<T>> builder) {
    return super.matches<Errored<T>>(builder);
  }
}

class FutureStateContainer<T> extends AsyncStateContainer<T> {
  final Future<T> future;

  FutureStateContainer(this.future);

  @override
  Future<Async<T>> load() {
    return future.toAsync();
  }
}

class StateStreamContainer<T> extends StateContainer<T> {
  final Stream<T> stream;
  final T initialState;
  StateStreamContainer(this.stream, this.initialState);

  Stream<T> init(_) => stream;
}

@immutable
abstract class StateContainerFactoryWidget<T> extends Widget {
  const StateContainerFactoryWidget({Key? key}) : super(key: key);

  StateContainer<T> _createContainer();

  @override
  Element createElement() {
    return StateContainerFactoryElement<T>(this);
  }

  Widget build(BuildContext context);
}

class StateContainerFactoryElement<T> extends ComponentElement {
  StateContainerFactoryElement(Widget widget) : super(widget);

  StateContainerFactoryWidget<T> get widget =>
      super.widget as StateContainerFactoryWidget<T>;
  late final StateContainer<T> container;

  @override
  void mount(Element? parent, Object? newSlot) {
    container = widget._createContainer();
    super.mount(parent, newSlot);
  }

  @override
  Widget build() {
    return StaticContainer(
      container: container,
      child: widget.build(this),
    );
  }
}

abstract class FutureStateWidget<T>
    extends StateContainerFactoryWidget<Async<T>> {
  const FutureStateWidget({Key? key}) : super(key: key);

  Future<T> get future;

  @override
  AsyncStateContainer<T> _createContainer() {
    return FutureStateContainer<T>(future);
  }
}

abstract class StateStreamWidget<T> extends StateContainerFactoryWidget<T> {
  const StateStreamWidget({Key? key}) : super(key: key);
  Stream<T> get stream;
  T get initialState;

  @override
  StateContainer<T> _createContainer() {
    return StateStreamContainer<T>(stream, initialState);
  }
}
